      SUBROUTINE KEARSLEY(X1,Y1,Z1,X2,Y2,Z2,NN,RMSDV,RR)
*
* declarations.
*
      IMPLICIT REAL(P-Q)
*
      INTEGER NN,I,J,K,L,HH
*
      REAL X1(NN),Y1(NN),Z1(NN),X2(NN),Y2(NN),Z2(NN)
      REAL XP,XM,YP,YM,ZP,ZM,Y
      REAL Q(4,4),RR(3,3),LAMBDA(4),PP(4),VV(4,4),WORK(80)
*
* initialise.
*
      DO 2 I=1,4
      PP(I)=0.0
      LAMBDA(I)=0.0
      DO 1 J=1,4
      Q(J,I)=0.0
      VV(J,I)=0.0
 1     CONTINUE
 2     CONTINUE
      DO 3 I=1,80
      WORK(I)=0.0
 3     CONTINUE
*
      Q11=0.0
      Q22=0.0
      Q33=0.0
      Q44=0.0
      Q12=0.0
      Q13=0.0
      Q14=0.0
      Q23=0.0
      Q24=0.0
      Q34=0.0
*
      IFAIL=0
*
* get coefficients.
*
      DO 4 I=1,NN
*
      XM=X1(I)-X2(I)
      YM=Y1(I)-Y2(I)
      ZM=Z1(I)-Z2(I)
      XP=X1(I)+X2(I)
      YP=Y1(I)+Y2(I)
      ZP=Z1(I)+Z2(I)
*
      Q11=Q11+(XM*XM+YM*YM+ZM*ZM)
      Q22=Q22+(XM*XM+YP*YP+ZP*ZP)
      Q33=Q33+(XP*XP+YM*YM+ZP*ZP)
      Q44=Q44+(XP*XP+YP*YP+ZM*ZM)
*
      Q12=Q12+(YP*ZM-YM*ZP)
      Q13=Q13+(XM*ZP-XP*ZM)
      Q14=Q14+(XP*YM-XM*YP)
      Q23=Q23+(XM*YM-XP*YP)
      Q24=Q24+(XM*ZM-XP*ZP)
      Q34=Q34+(YM*ZM-YP*ZP)
*
 4     CONTINUE
*
* fill arrays.
*
      Q(1,2)=Q12
      Q(2,1)=Q12
      Q(1,3)=Q13
      Q(3,1)=Q13
      Q(1,4)=Q14
      Q(4,1)=Q14
      Q(2,3)=Q23
      Q(3,2)=Q23
      Q(2,4)=Q24
      Q(4,2)=Q24
      Q(3,4)=Q34
      Q(4,3)=Q34
*
      Q(1,1)=Q11
      Q(2,2)=Q22
      Q(3,3)=Q33
      Q(4,4)=Q44
*
* call eigensolver.
*
      CALL NEWEIG(Q,LAMBDA,VV,4,4,4,WORK)
*
* error trap.
*
      IF (IFAIL.NE.0) WRITE (*,7) IFAIL
*
* get smallest eigenvalue.
*
      HH=1
      Y=LAMBDA(1)
      DO 5 I=2,4
      IF (LAMBDA(I).LT.Y) THEN
      Y=LAMBDA(I)
      HH=I
      ENDIF
 5     CONTINUE
*
      DO 6 I=1,4
      PP(I)=VV(I,HH)
 6     CONTINUE
*
      RMSDV=0.0
      IF (Y.GT.0.) THEN
      RMSDV=SQRT(LAMBDA(HH)/FLOAT(NN))
      ELSE
      RMSDV=SQRT(-LAMBDA(HH)/FLOAT(NN))
      ENDIF
*
      CALL QUATROT(RR,PP)
*
*
      RETURN
*
* format statements.
*
7     FORMAT (/' Eigensolver fails. Error flag',I2,'.'/)
      END
*
*-------------------------------------------------------------------------------
*
      SUBROUTINE QUATROT(R,Q)
*
* declarations.
*
      IMPLICIT INTEGER(A-Z)
*
      REAL R(3,3),Q(4),PQ(4),PS(4),PV(4,4),DET
*
* do it.
*
      CALL QNORM(Q,PQ)
*
      DO 2 I=1,4
      DO 1 J=1,4
      PV(J,I)=PQ(I)*PQ(J)
 1     CONTINUE
 2     CONTINUE
*
      DO 3 I=1,4
      PS(I)=PV(I,I)
 3     CONTINUE
*
      R(1,1)=PS(1)+PS(2)-PS(3)-PS(4)
      R(2,2)=PS(1)-PS(2)+PS(3)-PS(4)
      R(3,3)=PS(1)-PS(2)-PS(3)+PS(4)
*
      R(1,2)=2.0*(PV(2,3)+PV(1,4))
      R(2,1)=2.0*(PV(2,3)-PV(1,4))
      R(1,3)=2.0*(PV(2,4)-PV(1,3))
      R(3,1)=2.0*(PV(2,4)+PV(1,3))
      R(2,3)=2.0*(PV(3,4)+PV(1,2))
      R(3,2)=2.0*(PV(3,4)-PV(1,2))
*
* get determinant.
*
      DET=0.0
      DO 4 I=1,4
      DET=DET+PS(I)
 4     CONTINUE
      DET=DET**(1.0/3.0)
*
      RETURN
      END
*
*-------------------------------------------------------------------------------
*
      SUBROUTINE QNORM(Q,NQ)
*
* declarations.
*
      IMPLICIT INTEGER(A-Z)
*
      REAL Q(4),NQ(4),SUM
*
      SUM=0.0
      DO 1 I=1,4
      SUM=SUM+Q(I)*Q(I)
 1     CONTINUE
      SUM=SQRT(SUM)
*
      DO 2 I=1,4
      NQ(I)=Q(I)/SUM
 2     CONTINUE
*
      RETURN
      END
*-------------------------------------------------------------------------------
*
      SUBROUTINE NEWEIG(A,VALUE,VECTOR,M,IA,IV,W)
*
* Code modified to be standard FORTRAN 77.
*
*  given a real mxm symmetric matrix a = {aij} this routine finds all it
*  eigenvalues (lambda)i i=1,2,.....,m and eigenvectors xj i=1,2,...,m
*  i.e. finds the non-trivial solutions of ax=(lambda)x
*
*  subroutine neweig(a,value,x,m,ia,ix,w).
*
*  arguments set by the user -
*
*  a  is a real 2-dimensional array with first dimension ia. the user mu.
*     store the lower triangle of matrix a into the lower triangle of th.
*     array a. i.e. put a(ij) into a(i,j) for i>=j and i>=j.
*
*  m  is an integer and should be set to the order m of the matrix.
*
*  ia is an integer set to the first dimension of the array a, i.e. if t.
*     allocation for the array a was specified by.
*
*            dimension a(100,50).
*
*     then ia would be set to 100.
*
*  ix is an integer set to the first dimension of the 2-dimensional arra.
*
*  arguments set by the routine -.
*
*  a  the space above the diagonal of the array a is used by the routine.
*     work space, the lower triangle set by the user will remain unchang.
*     on return.
*
*  value is a real array in which the routine puts the eigenvalues (lamb.
*     i=1,2,...,m. these are not necessarily in any order.
*
*  x  is a real 2-dimensional array, with first dimension ix, containing.
*     eigenvectors. the components of the eigenvector x(i) corresponding.
*     eigenvalue (lambda)i (in value(i)) are placed in x(j,i) j=1,2,...,.
*     the eigenvectors are normalized si that xt(i)x(i)=1 i=1,2,...,m.
*
*  w  is a real array used by the routine for work space. it must have.
*     dimension at least 5*m.
*
      REAL A(IA,M),VALUE(M),VECTOR(IV,M),W(1)
*
      M1=M+1
      W(1)=A(1,1)
      IF (M-2) 3,1,2
 1     CONTINUE
      W(2)=A(2,2)
      W(4)=A(2,1)
      GOTO 3
*
 2     CONTINUE
      CALL MC04B(A,W,W(M1),M,IA,W(M+M1))
 3     CONTINUE
      CALL EA08C(W,W(M1),VALUE,VECTOR,M,IV,W(M+M1))
      IF (M.LE.2) RETURN
      DO 8 L=1,M
      DO 7 II=3,M
      I=M-II+1
      IF (W(M1+I)) 4,7,4
 4     CONTINUE
      PP=0.
      I1=I+1
      DO 5 K=I1,M
      PP=PP+A(I,K)*VECTOR(K,L)
 5     CONTINUE
      PP=PP/(A(I,I+1)*W(M1+I))
      DO 6 K=I1,M
      VECTOR(K,L)=VECTOR(K,L)+PP*A(I,K)
 6     CONTINUE
 7     CONTINUE
 8     CONTINUE
*
      RETURN
      END
*
*-------------------------------------------------------------------------------
*
      SUBROUTINE MC04B(A,ALPHA,BETA,M,IA,Q)
*
      DIMENSION A(IA,1),ALPHA(1),BETA(1),Q(1)
*
      ALPHA(1)=A(1,1)
      DO 2 J=2,M
      J1=J-1
      DO 1 I=1,J1
      A(I,J)=A(J,I)
 1     CONTINUE
      ALPHA(J)=A(J,J)
 2     CONTINUE
      M1=M-1
      M2=M-2
      DO 17 I=1,M2
      PP=0.0
      I1=I+1
      DO 3 J=I1,M
      PP=PP+A(I,J)**2
 3     CONTINUE
      PP1=SQRT(PP)
      IF (A(I,I+1)) 5,4,4
 4     CONTINUE
      BETA(I+1)=-PP1
      GOTO 6
*
 5     CONTINUE
      BETA(I+1)=PP1
 6     CONTINUE
      IF (PP) 17,17,7
 7     CONTINUE
      H=PP-BETA(I+1)*A(I,I+1)
      A(I,I+1)=A(I,I+1)-BETA(I+1)
      DO 12 KI=I1,M
      QJ=0.0
      DO 8 KJ=I1,KI
      QJ=QJ+A(KJ,KI)*A(I,KJ)
 8     CONTINUE
      IF (KI-M) 9,11,11
 9     CONTINUE
      I2=KI+1
      DO 10 KJ=I2,M
      QJ=QJ+A(KI,KJ)*A(I,KJ)
 10    CONTINUE
 11    CONTINUE
      Q(KI)=QJ/H
 12    CONTINUE
      BIGK=0.0
      DO 13 KJ=I1,M
      BIGK=BIGK+A(I,KJ)*Q(KJ)
 13    CONTINUE
      BIGK=BIGK/(2.0*H)
      DO 14 KJ=I1,M
      Q(KJ)=Q(KJ)-BIGK*A(I,KJ)
 14    CONTINUE
      DO 16 KI=I1,M
      DO 15 KJ=KI,M
      A(KI,KJ)=A(KI,KJ)-Q(KI)*A(I,KJ)-Q(KJ)*A(I,KI)
 15    CONTINUE
 16    CONTINUE
 17    CONTINUE
      DO 18 I=2,M
      H=ALPHA(I)
      ALPHA(I)=A(I,I)
      A(I,I)=H
 18    CONTINUE
      BETA(M)=A(M-1,M)
*
      RETURN
      END
*
*-------------------------------------------------------------------------------
*
      SUBROUTINE EA08C(A,B,VALUE,VEC,M,IV,W)
*
* Declarations.
*
      REAL A(M),B(M),VALUE(M),VEC(1),W(1)
      EPS=1.0E-5
      A34=0.0
*
* this uses qr iteration to find the eigenvalues and eigenvectors.
* of the symmetric tridiagonal matrix whose diagonal elements are.
* a(i),i=1,m and off-diagonal elements are b(i),i=2,m.  the array.
* w is used for workspace and must have dimension at least 2*m.
* we treat vec as if it had dimensions (iv,m).
*
      CALL EA09C(A,B,W(M+1),M,W)
*
* set vec to the identity matrix.
*
      DO 2 I=1,M
      VALUE(I)=A(I)
      W(I)=B(I)
      K=(I-1)*IV+1
      L=K+M-1
      DO 1 J=K,L
      VEC(J)=0.
 1     CONTINUE
      VEC(K+I-1)=1.
 2     CONTINUE
      ITER=0
      IF (M.EQ.1) RETURN
      N2=M
*
*  each qr iteration is performed of rows and columns n1 to n2.
*
 3     CONTINUE
      DO 4 II=2,N2
      N1=2+N2-II
      IF (ABS(W(N1)).LE.(ABS(VALUE(N1-1))+ABS(VALUE(N1)))*EPS) GOTO 5
 4     CONTINUE
      N1=1
 5     CONTINUE
      IF (N2.NE.N1) GOTO 6
      N2=N2-1
      IF (N2-1) 9,9,3
 6     CONTINUE
      ROOT=W(M+N2)
      ITER=ITER+1
      N2M1=N2-1
      A22=VALUE(N1)
      A12=A22-ROOT
      A23=W(N1+1)
      A13=A23
      DO 8 I=N1,N2M1
      A33=VALUE(I+1)
      IF (I.NE.N2M1) A34=W(I+2)
      S=SIGN(SQRT(A12*A12+A13*A13),A12)
      SI=A13/S
      CO=A12/S
      JK=I*IV+1
      J1=JK-IV
      J2=J1+MIN0(M,I+ITER)-1
      DO 7 JI=J1,J2
      V1=VEC(JI)
      V2=VEC(JK)
      VEC(JI)=V1*CO+V2*SI
      VEC(JK)=V2*CO-V1*SI
      JK=JK+1
 7     CONTINUE
      IF (I.NE.N1) W(I)=S
      A11=CO*A22+SI*A23
      A12=CO*A23+SI*A33
      A13=SI*A34
      A21=CO*A23-SI*A22
      A22=CO*A33-SI*A23
      A23=CO*A34
      VALUE(I)=A11*CO+A12*SI
      A12=-A11*SI+A12*CO
      W(I+1)=A12
      A22=A22*CO-A21*SI
 8     CONTINUE
      VALUE(N2)=A22
      GOTO 3
*
* rayleigh quotient.
*
 9     CONTINUE
      DO 11 J=1,M
      K=(J-1)*IV
      XX=VEC(K+1)**2
      XAX=XX*A(1)
      DO 10 I=2,M
      XX=XX+VEC(K+I)**2
      XAX=XAX+VEC(K+I)*(2.*B(I)*VEC(K+I-1)+A(I)*VEC(K+I))
 10    CONTINUE
      VALUE(J)=XAX/XX
 11    CONTINUE
*
      RETURN
      END
*
*-------------------------------------------------------------------------------
*
      SUBROUTINE EA09C(A,B,VALUE,M,OFF)
*
* Declarations.
*
      REAL A(M),B(M),VALUE(M),OFF(M),A34/0./,EPS/0.6E-7/
*
      VALUE(1)=A(1)
      IF (M.EQ.1) RETURN
      DO 1 I=2,M
      VALUE(I)=A(I)
      OFF(I)=B(I)
 1     CONTINUE
*
*  each qr iteration is performed of rows and columns n1 to n2.
*
      N2=M
 2     CONTINUE
      IF (N2.LE.1) RETURN
      DO 3 II=2,N2
      N1=2+N2-II
      IF (ABS(OFF(N1)).LE.(ABS(VALUE(N1-1))+ABS(VALUE(N1)))*EPS) GOTO 4
 3     CONTINUE
      N1=1
 4     CONTINUE
      IF (N2.NE.N1) GOTO 5
      N2=N2-1
      GOTO 2
*
*  root  is the eigenvalue of the bottom 2*2 matrix that is nearest.
*  to the last matrix element and is used to accelerate the.
*  convergence.
*
 5     CONTINUE
      BB=(VALUE(N2)-VALUE(N2-1))*0.5
      CC=OFF(N2)*OFF(N2)
      SBB=1.
      IF (BB.LT.0.0) SBB=-1.0
      ROOT=VALUE(N2)+CC/(BB+SBB*SQRT(BB*BB+CC))
      N2M1=N2-1
      A22=VALUE(N1)
      A12=A22-ROOT
      A23=OFF(N1+1)
      A13=A23
      DO 6 I=N1,N2M1
      A33=VALUE(I+1)
      IF (I.NE.N2M1) A34=OFF(I+2)
      S=SQRT(A12*A12+A13*A13)
      SI=A13/S
      CO=A12/S
      IF (I.NE.N1) OFF(I)=S
      A11=CO*A22+SI*A23
      A12=CO*A23+SI*A33
      A13=SI*A34
      A21=CO*A23-SI*A22
      A22=CO*A33-SI*A23
      A23=CO*A34
      VALUE(I)=A11*CO+A12*SI
      A12=-A11*SI+A12*CO
      OFF(I+1)=A12
      A22=A22*CO-A21*SI
 6     CONTINUE
      VALUE(N2)=A22
      GOTO 2
*
      END
*
*-----------------------
